#!/usr/bin/env python3

import argparse
from rm2trash.core import logging
from rm2trash.commands import subcommands


def main():
    parser = argparse.ArgumentParser(description="Simple trash commands: rm2trash.")
    parser.add_argument(
        "-l", "--log_level", type=str, default="INFO", help="Log level."
    )
    sub_parsers = parser.add_subparsers(
        title="subcommands", description="All subcommands", required=True
    )

    # Remove
    remove = sub_parsers.add_parser("remove", aliases=["rm"])
    remove.add_argument(
        "files_or_dirs", type=str, nargs="+", help="files or directories to remove."
    )
    remove.set_defaults(func=subcommands.remove)

    # Delete
    delete = sub_parsers.add_parser("delete", aliases=["del"])
    del_group = delete.add_mutually_exclusive_group(required=True)
    del_group.add_argument(
        "-i", "--indexes", type=int, nargs="+", help="trash item index to delete."
    )
    del_group.add_argument(
        "-p", "--pattern", type=str, help="pattern for trash item name to match."
    )
    delete.set_defaults(func=subcommands.delete)

    # Empty
    empty = sub_parsers.add_parser("empty")
    empty.set_defaults(func=subcommands.empty)

    # Restore
    restore = sub_parsers.add_parser("restore")
    restore_group = restore.add_mutually_exclusive_group(required=True)
    restore_group.add_argument(
        "-i", "--indexes", type=int, nargs="+", help="trash item index to restore."
    )
    restore_group.add_argument(
        "-p", "--pattern", type=str, help="pattern for trash item name to match."
    )
    restore.set_defaults(func=subcommands.restore)

    # List
    list_command = sub_parsers.add_parser("list", aliases=["ls"])
    list_command.set_defaults(func=subcommands.list_command)

    # Path
    trash_path = sub_parsers.add_parser("trash-path")
    trash_path.set_defaults(func=subcommands.print_trash_path)
    data_path = sub_parsers.add_parser("data-path")
    data_path.set_defaults(func=subcommands.print_data_path)

    args = parser.parse_args()
    logging.set_level(args.log_level)
    args.func(args)


if __name__ == "__main__":
    main()



import sys
import logging


def get_logger():
    logger = logging.getLogger("RM2TRASH")
    if len(logger.handlers) == 0:
        # Prevent logging from propagating to the root logger
        logger.propagate = 0
        console_handler = logging.StreamHandler(sys.stderr)
        logger.addHandler(console_handler)
        formatter = logging.Formatter("[%(asctime)s] [%(levelname)s] : %(message)s")
        console_handler.setFormatter(formatter)
    return logger


def set_level(level):
    if isinstance(level, str):
        get_logger().setLevel(level.upper())
    else:
        get_logger().setLevel(level)


debug = get_logger().debug
info = get_logger().info
warning = get_logger().warning
error = get_logger().error
fatal = get_logger().fatal



from cmath import log
import os
import platform
from pathlib import Path
from rm2trash.core import logging


RM2TRASH_TRASH_PATH = os.environ.get("RM2TRASH_TRASH_PATH", None)
if RM2TRASH_TRASH_PATH is None:
    _system = platform.system()
    if _system == "Darwin":
        RM2TRASH_TRASH_PATH = Path.home() / ".Trash"
    elif _system == "Linux":
        RM2TRASH_TRASH_PATH = Path.home() / ".local/share/Trash"
    else:
        raise ValueError(
            "RM2TRASH_TRASH_PATH not set. No default trash dir for OS {}".format(
                _system
            )
        )
else:
    RM2TRASH_TRASH_PATH = Path(RM2TRASH_TRASH_PATH)
if not RM2TRASH_TRASH_PATH.exists():
    logging.info("{} not exists, will make this directory.", RM2TRASH_TRASH_PATH)
    RM2TRASH_TRASH_PATH.mkdir(parents=True, exist_ok=True)

RM2TRASH_DATA_PATH = os.environ.get("RM2TRASH_DATA_PATH")
RM2TRASH_DATA_PATH = Path(RM2TRASH_DATA_PATH or Path.home() / ".local/share/rm2trash")
if not RM2TRASH_DATA_PATH.exists():
    logging.info("{} not exists, will make this directory.", RM2TRASH_DATA_PATH)
    RM2TRASH_DATA_PATH.mkdir(parents=True, exist_ok=True)



from termcolor import colored
from rm2trash.core.trash_manager import TrashManager
from rm2trash.core import global_variables


def need_confirm(func):
    def res(args):
        user_input = input(
            colored(
                "YOU CANNOT UNDO THIS!!! ARE YOU SURE? [y/n] ",
                color="red",
                attrs=["bold"],
            )
        )
        if user_input.upper() not in ["Y", "YES"]:
            return
        return func(args)

    return res


@need_confirm
def delete(args):
    tm = TrashManager()
    if args.indexes is not None:
        tm.delete_in_trash_by_indexes(args.indexes)
    else:
        tm.delete_in_trash_by_name_pattern(args.pattern)


def remove(args):
    tm = TrashManager()
    tm.move_files_or_dirs_to_trash(args.files_or_dirs)


@need_confirm
def empty(_):
    tm = TrashManager()
    tm.empty_trash()


def list_command(_):
    tm = TrashManager()
    tm.list_data()


def restore(args):
    tm = TrashManager()
    if args.indexes is not None:
        tm.restore_from_trash_by_indexes(args.indexes)
    else:
        tm.restore_from_trash_by_name_pattern(args.pattern)


def print_trash_path(_):
    print(global_variables.RM2TRASH_TRASH_PATH.absolute(), end="")


def print_data_path(_):
    print(global_variables.RM2TRASH_DATA_PATH.absolute(), end="")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rm2trash', 'rm2trash.commands', 'rm2trash.core']

package_data = \
{'': ['*']}

install_requires = \
['termcolor>=1.1.0,<2.0.0']

entry_points = \
{'console_scripts': ['rm2trash = rm2trash.commands.rm2trash:main']}

setup_kwargs = {
    'name': 'rm2trash',
    'version': '0.1.1',
    'description': 'Simple trash commands.',
    'long_description': None,
    'author': 'Kuangye Chen',
    'author_email': 'chen.kuangye@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import re
import json
import glob
import shutil
import datetime
from pathlib import Path
from typing import List, Dict
from rm2trash.core import logging
from rm2trash.core.global_variables import RM2TRASH_TRASH_PATH, RM2TRASH_DATA_PATH


class TrashManager:
    def __init__(self) -> None:
        self.database_file = RM2TRASH_DATA_PATH / "rm2trash_db.json"
        self.time_format = "%Y-%m-%d %H:%M:%S"
        self.data = self._read_data()

    def search_item(self, pattern: str) -> List[int]:
        res = []
        for i, data in enumerate(self.data):
            if re.search(pattern, data["name"]):
                res.append(i)
        return res

    def move_files_or_dirs_to_trash(self, files_or_dirs: List[str]) -> None:
        path_lists = []
        for f_or_d in files_or_dirs:
            path_lists += glob.glob(f_or_d)

        count = 0
        for f_or_d in path_lists:
            source = Path(f_or_d)
            if not source.exists:
                logging.warning("{} not exists.".format(source))
                continue

            if source.is_symlink():
                logging.info("Unlink {} -> {}.".format(source, source.readlink()))
                source.unlink()
                continue

            target = RM2TRASH_TRASH_PATH / source.name
            suffix_int = 1
            while target.exists():
                target = RM2TRASH_TRASH_PATH / "{}.{}".format(source.name, suffix_int)
                suffix_int += 1

            shutil.move(source, target)
            logging.info("Move {} to trash.".format(str(source)))

            new_item = dict(
                name=source.name,
                source_path=str(source.absolute()),
                trash_path=str(target.absolute()),
                datetime=datetime.datetime.now().strftime(self.time_format),
            )
            self.data.append(new_item)
            count += 1
        logging.info("{} item(s) moved to trash.".format(count))
        self._dump_data(self.data)

    def delete_in_trash_by_indexes(self, indexes: List[int]) -> None:
        index_to_delete = []
        for index in indexes:
            if index < 0 or index >= len(self.data):
                logging.warning(
                    "Invalid index {}, trash currently has {} items.".format(
                        index, len(self.data)
                    )
                )
                continue

            data = self.data[index]
            target = Path(data["trash_path"])
            if not target.exists():
                logging.warning(
                    "Trash to delete {} not exists. Do nothing.".format(target)
                )
            else:
                if target.is_dir():
                    shutil.rmtree(target)
                else:
                    target.unlink()
                logging.info("Permanently delete {}.".format(data["trash_path"]))
            index_to_delete.append(index)
        logging.info("{} item(s) deleted.".format(len(index_to_delete)))
        self.data = [
            data for i, data in enumerate(self.data) if i not in index_to_delete
        ]
        self._dump_data(self.data)

    def delete_in_trash_by_name_pattern(self, pattern: str) -> None:
        indexes = self.search_item(pattern)
        self.delete_in_trash_by_indexes(indexes)

    def empty_trash(self) -> None:
        self.delete_in_trash_by_indexes(list(range(len(self.data))))

    def restore_from_trash_by_indexes(self, indexes: List[int]) -> None:
        index_to_delete = []
        for index in indexes:
            if index < 0 or index >= len(self.data):
                logging.warning(
                    "Invalid index {}, trash currently has {} items.".format(
                        index, len(self.data)
                    )
                )
                continue

            data = self.data[index]
            source = Path(data["source_path"])
            target = Path(data["trash_path"])
            if not target.exists():
                logging.warning(
                    "Trash item to restore {} not exists. Remove item.".format(target)
                )
                index_to_delete.append(index)
                continue
            if source.exists():
                logging.warning(
                    "Source to restore {} exists. Do nothing.".format(source)
                )
                continue

            shutil.move(target, source)
            index_to_delete.append(index)
            logging.info("Restore {}.".format(source))
        logging.info("{} item(s) restored.".format(len(index_to_delete)))
        self.data = [
            data for i, data in enumerate(self.data) if i not in index_to_delete
        ]
        self._dump_data(self.data)

    def restore_from_trash_by_name_pattern(self, pattern: str) -> None:
        indexes = self.search_item(pattern)
        self.restore_from_trash_by_indexes(indexes)

    def list_data(self) -> None:
        for i, data in enumerate(self.data):
            print(
                '{}: name="{}", source="{}", rm_time="{}"'.format(
                    i, data["name"], data["source_path"], data["datetime"]
                )
            )

    def _read_data(self) -> List[Dict]:
        if not self.database_file.exists():
            return []

        with open(self.database_file, "r") as f:
            return json.load(f)

    def _dump_data(self, data: list) -> None:
        with open(self.database_file, "w") as f:
            json.dump(data, f, indent=2)
